<?php
/*
* 2007-2024 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class TcbpayCardValidation {
    public $cc_type;
    public $cc_name;
    public $cc_number;
    public $cc_expiry_month;
    public $cc_expiry_year;

    public function validate($number, $expiry_m, $expiry_y) {
        $this->cc_number = preg_replace('/[^0-9]/', '', $number);

        if (preg_match('/^4[0-9]{12,18}$/', $this->cc_number)) {
            $this->cc_type = 'VI';
            $this->cc_name = 'Visa';
        } elseif (preg_match('/^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$/', $this->cc_number)) {
            $this->cc_type = 'MC';
            $this->cc_name = 'MasterCard';
        } elseif (preg_match('/^3(4|7)[0-9]{13}/', $this->cc_number)) {
            $this->cc_type = 'AE';
            $this->cc_name = 'American Express';
        } elseif (preg_match('/^(6011((0|9|[2-5])[0-9]{11,14}|(74|7[7-9]|8[6-9])[0-9]{10,13})|6(4[4-9][0-9]{13,16}|5[0-9]{14,17}))/', $this->cc_number)) {
            $this->cc_type = 'DI';
            $this->cc_name = 'Discover';
        } elseif (preg_match('/^35(2[8-9][0-9]{12,15}|[3-8][0-9]{13,16})$/', $this->cc_number)) {
            $this->cc_type = 'JCB';
            $this->cc_name = 'JCB';
        } elseif (preg_match('/^\\d+$/', $this->cc_number)) {
            $this->cc_type = 'OT';
            $this->cc_name = 'Other';
        } else {
            return -1;
        }

        if (is_numeric($expiry_m) && ($expiry_m > 0) && ($expiry_m < 13)) {
            $this->cc_expiry_month = $expiry_m;
        } else {
            return -2;
        }

        $current_year = date('Y');
        $expiry_y = substr($current_year, 0, 2) . $expiry_y;
        if (is_numeric($expiry_y) && ($expiry_y >= $current_year) && ($expiry_y <= ($current_year + 10))) {
            $this->cc_expiry_year = $expiry_y;
        } else {
            return -3;
        }

        if ($expiry_y == $current_year) {
            if ($expiry_m < date('n')) {
                return -4;
            }
        }

        return $this->is_valid();
    }

    public function is_valid() {
        $cardNumber = strrev($this->cc_number);
        $numSum = 0;

        for ($i=0; $i<strlen($cardNumber); $i++) {
            $currentNum = substr($cardNumber, $i, 1);

            // Double every second digit
            if ($i % 2 == 1) {
                $currentNum *= 2;
            }

            // Add digits of 2-digit numbers together
            if ($currentNum > 9) {
                $firstNum = $currentNum % 10;
                $secondNum = ($currentNum - $firstNum) / 10;
                $currentNum = $firstNum + $secondNum;
            }

            $numSum += $currentNum;
        }

        // If the total has no remainder it's OK
        return ($numSum % 10 == 0);
    }

    public function getCvvRegEx()
    {
        $verificationExpList = [
            'VI' => '/^[0-9]{3}$/',
            'MC' => '/^[0-9]{3}$/',
            'AE' => '/^[0-9]{3,4}$/',
            'DI' => '/^[0-9]{3}$/',
            'OT' => '/^[0-9]{3,4}$/',
            'JCB' => '/^[0-9]{3,4}$/'
        ];

        return $verificationExpList;
    }
}