<?php
/*
* 2007-2024 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class TcbpayGateway {
    const API_LIVE_URL = 'https://gateway2.tcbpay.com/mobile_plugin/';
    const API_TEST_URL = 'https://gateway2-stg.tcbpay.com/mobile_plugin/';

    public function decodeCard($accountNumber = '') {
        $params = array(
            'accountNumber' => $accountNumber
        );

        return $this->gatewayRequest('proxynizationED.php', $params);
    }

    public function sendRequest($params = array()) {
        $rparams = array(
            'client_ip' => $_SERVER['REMOTE_ADDR'],
            'source' => 'API_PRESTASHOP'
        );
        $rparams = array_merge($rparams, $params);

        return $this->gatewayRequest('sale.php', $rparams);
    }

    private function gatewayRequest($path='', $params = array()) {
        $rparams = array(
            'username' => Configuration::get('TCBPAY_USERNAME'),
            'api_key' => Configuration::get('TCBPAY_PASSWORD')
        );

        $rparams = array_merge($rparams, $params);

        $gmtime = strtotime(gmdate('Y-m-d H:i:s'));

        $headers = [
            'hash_timestamp: '. $gmtime,
            'hash: '. md5($rparams['username']."|".$rparams['api_key']."|".Configuration::get('TCBPAY_SECRETKEY')."|".$gmtime)
        ];

        $url = self::API_LIVE_URL . $path;
        if (Configuration::get('TCBPAY_TESTMODE')) {
            $url = self::API_TEST_URL . $path;
        }

        $post_query =  http_build_query($rparams);

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, true); 
        curl_setopt($ch,CURLOPT_POSTFIELDS, $post_query);
        $response = curl_exec($ch);

        curl_close ($ch);

        $result = ['success' => false, 'resp' => ''];
        $respObj = json_decode($response);

        if (isset($respObj) && !empty($respObj) && isset($respObj->response)) {
            $resp = $respObj->response;
            $result['resp'] = $resp;
            if($resp->code == '00') {
                $result['success'] = true;
            }
        }

        return $result;
    }

}